-- << dllsetup.e >> -- opens dll files and links requested routines
--  ver 1.1, 3-27-98  /  Colin Taylor <cetaylor@compuserve.com>

integer file, line_len, last
sequence dll_db, routine_db, win32_db, constants_db, routine_list

-- list of dlls recognized by dllsetup.e
dll_db = {"user32", "gdi32", "winmm", "comdlg32"}

-- database of routines contained in dll files
-- {{routine_name, routine_type, dll_name, dll_routine_name, var_list} ... }
routine_db = {  
    -- user32.dll routines:
    {"BeginPaint", "func", "user32", "BeginPaint", "{C_INT, C_POINTER}, C_INT"},
    {"CreateWindow", "func", "user32", "CreateWindowExA", "{C_INT, C_INT, " &
	"C_INT, C_INT, C_INT, C_INT, C_INT, C_INT, C_INT, C_INT, C_INT, " &
	"C_INT}, C_INT"},
    {"DefWindowProc", "func", "user32", "DefWindowProcA", "{C_INT, C_INT, " &
	"C_INT, C_INT}, C_INT"},
    {"DispatchMessage", "proc", "user32", "DispatchMessageA", "{C_INT}"},
    {"DrawText", "proc", "user32", "DrawTextA", "{C_INT, C_INT, " &
	"C_INT, C_INT, C_INT}"},
    {"EndPaint", "proc", "user32", "EndPaint", "{C_INT, C_INT}"},
    {"GetActiveWindow", "func", "user32", "GetActiveWindow", "{},C_INT"},
    {"GetClientRect", "proc", "user32", "GetClientRect", "{C_INT, C_POINTER}"},
    {"GetDC", "func", "user32", "GetDC", "{C_INT},C_INT"},
    {"GetMessage", "func", "user32", "GetMessageA", "{C_INT, C_INT, C_INT, " &
	"C_INT}, C_INT"},
    {"InvalidateRect", "func", "user32", "InvalidateRect", "{C_INT, C_INT, " &
	"C_INT}, C_INT"},
    {"IsDialogMessage", "func", "user32", "IsDialogMessage", "{C_INT, " &
	"C_INT}, C_INT"},
    {"KillTimer", "proc", "user32", "KillTimer", "{C_INT, C_INT}"},
    {"LoadIcon", "func", "user32", "LoadIconA", "{C_POINTER, C_INT}, C_INT"},
    {"LoadCursor", "func", "user32", "LoadCursorA", "{C_POINTER, C_INT}, " &
	"C_INT"},
    {"MessageBeep", "func", "user32", "MessageBeep", "{C_UINT}, C_INT"},
    {"PostMessage", "func", "user32", "PostMessageA", "{C_INT, C_UINT, " &
	"C_INT,C_LONG},C_UINT"},
    {"PostQuitMessage", "proc", "user32", "PostQuitMessage", "{C_INT}"},
    {"RegisterClassEx", "func", "user32", "RegisterClassExA", "{C_POINTER}, " &
	"C_INT"},
    {"RegisterWindowMessage", "func", "user32", "RegisterWindowMessageA", 
	"{C_POINTER},C_UINT"},
    {"ReleaseDC", "proc", "user32", "ReleaseDC", "{C_INT,C_INT}"},
    {"SendMessage", "func", "user32", "SendMessageA", "{C_INT, C_INT, C_INT, " &
	"C_INT},C_INT"},
    {"SetFocus", "func", "user32", "SetFocus", "{C_INT},C_INT"},
    {"SetForegroundWindow", "func", "user32", "SetForegroundWindow", 
	"{C_INT}, C_INT"},            
    {"SetScrollInfo", "func", "user32", "SetScrollInfo", "{C_INT, C_INT, " &
	"C_INT, C_INT}, C_INT"},
    {"SetTimer", "proc", "user32", "SetTimer", "{C_INT, C_INT, C_INT, C_INT}"},
    {"ShowWindow", "proc", "user32", "ShowWindow", "{C_INT, C_INT}"},
    {"TranslateMessage", "proc", "user32", "TranslateMessage", "{C_INT}"},
    {"UpdateWindow", "proc", "user32", "UpdateWindow", "{C_INT}"},
    -- gdi32.dll routines:
    {"CreateDC", "func", "gdi32", "CreateDCA", "{C_POINTER, C_POINTER, " &
	"C_POINTER, C_POINTER}, C_LONG"},
    {"CreatePen", "func", "gdi32", "CreatePen", "{C_LONG, C_LONG, C_LONG}, " &
	"C_LONG"},
    {"GetStockObject", "func", "gdi32", "GetStockObject", "{C_INT}, C_INT"},
    {"GetTextMetrics", "func", "gdi32", "GetTextMetricsA", "{C_UINT, " &
	"C_POINTER}, C_INT"},
    {"LineTo", "func", "gdi32", "LineTo", "{C_LONG, C_LONG, C_LONG}, C_LONG"},
    {"MoveToEx", "func", "gdi32", "MoveToEx", "{C_LONG, C_LONG, C_LONG, " &
	"C_POINTER}, C_LONG"},
    {"SelectObject", "func", "gdi32", "SelectObject", "{C_INT, C_INT}, C_INT"},
    {"SetTextColor", "func", "gdi32", "SetTextColor", "{C_INT, C_UINT}, " &
	"C_UINT"},
    {"TextOut", "func", "gdi32", "TextOutA", "{C_INT, C_INT, C_INT, " &
	"C_POINTER, C_INT},C_INT"},
    -- winmm.dll routines:
    {"PlaySound", "proc", "winmm", "PlaySound", "{C_INT, C_INT, C_INT}"},
    -- comdlg32.dll routines:
    {"", "", "", "", ""}
}

-- database of general win32 constants
-- {"name = value" ... }
win32_db = {
    -- window class structure
    "cbSize = 0", 
    "style = 4", 
    "lpfnWndProc = 8", 
    "cbClsExtra = 12", 
    "cbWndExtra = 16", 
    "hInstance = 20", 
    "hIcon = 24", 
    "hCursor = 28", 
    "hbrBackground = 32", 
    "lpszMenuName = 36", 
    "lpszClassName = 40", 
    "hIconSm = 44", 
    -- memory allocated for windows class
    "SIZE_OF_WNDCLASS = 48", 
    -- memory allocated for windows messages
    "SIZE_OF_MESSAGE = 40", 
    -- class styles
    "CS_VREDRAW = 1", 
    "CS_HREDRAW = 2", 
    -- windows messages
    "WM_CREATE = #0001", 
    "WM_DESTROY = #0002", 
    "WM_SIZE = #0005", 
    "WM_SETFOCUS = #0007", 
    "WM_KILLFOCUS = #0008", 
    "WM_PAINT = #000F", 
    "WM_KEYDOWN = #0100", 
    "WM_COMMAND = #0111", 
    "WM_TIMER = #0113", 
    "WM_HSCROLL = #0114", 
    "WM_VSCROLL = #0115"
}

-- sequence of databases of dll-specific constants
-- {{database1}, {database2} ... }  -  each database belongs to the 
--   corresponding dll file in the dll list (dll_db).
constants_db = {
    -- constants used by user32.dll
    -- {"name = value" ... }
    {
    -- ShowWindow() commands
    "SW_HIDE = 0", 
    "SW_SHOWNORMAL = 1", 
    "SW_SHOWMINIMIZED= 2",
    "SW_SHOWMAXIMIZED = 3", 
    -- DrawText() format flags
    "DT_LEFT = #0", 
    "DT_CENTER = #1", 
    "DT_RIGHT = #2", 
    "DT_VCENTER = #4", 
    "DT_BOTTOM = #8", 
    "DT_WORDBREAK = #10", 
    "DT_SINGLELINE = #20", 
    "DT_EXPANDTABS = #40", 
    "DT_TABSTOP = #80", 
    "DT_NOCLIP = #100", 
    "DT_EXTERNALLEADING = #200", 
    "DT_CALCRECT = #400",
    "DT_NOPREFIX = #800", 
    "DT_INTERNAL = #1000", 
    "DT_EDITCONTROL = #2000",
    "DT_PATH_ELLIPSIS = #4000", 
    "DT_END_ELLIPSIS = #8000", 
    "DT_MODIFYSTRING = #10000", 
    "DT_RTLREADING = #20000", 
    "DT_WORD_ELLIPSIS = #40000", 
    -- windows style
    "WS_BORDER = #00800000", 
    "WS_CAPTION = #00C00000", 
    "WS_CHILD = #40000000",
    "WS_DLGFRAME = #00400000", 
    "WS_HSCROLL = #00100000", 
    "WS_MAXIMIZEBOX = #00010000", 
    "WS_MINIMIZEBOX = #00020000",
    "WS_OVERLAPPED = #00000000",
    "WS_SYSMENU = #00080000", 
    "WS_THICKFRAME = #00040000", 
    "WS_VISIBLE = #10000000", 
    "WS_VSCROLL = #00200000", 
    "WS_OVERLAPPEDWINDOW = or_all({WS_OVERLAPPED, WS_CAPTION, WS_SYSMENU,\n\t" &
	"WS_THICKFRAME, WS_MINIMIZEBOX, WS_MAXIMIZEBOX})",
    -- standard icon IDs
    "IDI_APPLICATION = 32512", 
    "IDC_ARROW = 32512", 
    -- create window
    "CW_USEDEFAULT = #80000000"
    },
    -- constants used by gdi32.dll
    -- {"name = value" ... }
    {
    -- stock logical objects
    "WHITE_BRUSH = 0", 
    "WHITE_PEN = 6",
    "BLACK_PEN = 7", 
    -- pen style
    "PS_SOLID = 0"
    },
    -- constants used by winmm.dll
    -- {"name = value" ... }
    {
    -- flag values for PlaySound()
    "SND_ASYNC = #00001",
    "SND_FILENAME = #00020000" 
    },
    -- constants used by comdlg32.dll
    -- {"name = value" ... }
    {}
}

procedure wrap(integer word_len)
-- wraps long lines
    line_len = line_len+word_len
    if line_len > 80 then
	puts(file, "\n\t")
	line_len = word_len+8
    end if
end procedure  -- wrap

procedure put_list(sequence list)
-- prints a list to file with word wrap
    for i = 1 to last-1 do
	wrap(length(list[i])+2)
	puts(file, list[i] & ", ")
    end for
    wrap(length(list[last]))
    puts(file, list[last] & "\n\n")
end procedure  -- put_list

function dll_list(sequence list)
-- prepare list of dll files needed
    integer found
    sequence newlist
    newlist = {}
    for i = 1 to length(list) do
	found = 0
	for j = 1 to length(routine_db) do
	    if not compare(list[i], routine_db[j][1]) then
		found = 1
		if not find(routine_db[j][3], newlist) then
		    if find(routine_db[j][3], dll_db) then
			newlist = append(newlist, routine_db[j][3])
		    else
			puts(1, "*\"" & routine_db[j][3] & 
				".dll\" not set up in dllsetup.e*\n")
		    end if
		end if
		exit
	    end if
	end for
	if not found then
	    puts(1, "*routine \"" & list[i] & "\" not found in database*\n")
	end if
    end for
    return newlist
end function  -- dll_list

global procedure dllsetup(sequence file_name, sequence list)
-- generates an include file
    sequence dlls
    routine_list = list
    puts(1, "running dllsetup...\n")
    file = open(file_name, "r")
    if file != -1 then
	-- backup existing include file
	close(file)
	puts(1, "backing up previous " & file_name)
	system("copy " & file_name & " ds.bak", 2)
    end if
    file = open(file_name, "w")
    if file = -1 then
	puts(1, "*couldn't open " & file_name & "*\n")
	abort(1)
    end if
    puts(file, "-- << "& file_name &" >> -- win32 header file generated " &
	"by dllsetup.e\n\ninclude dll.e\n\n")
    last = length(win32_db)
    puts(file, "-- win32 global constants\nglobal constant ")
    line_len = 16
    put_list(win32_db)
    puts(file, "global integer WM_FIND\nWM_FIND = 0\n\n")
    puts(file, "integer error\nerror = 0\n\n")
    puts(file, "global function or_all(sequence s)\n-- or together all " &
	"elements of s\n    atom result\n    result = 0\n    for i = 1 " &
	"to length(s) do\n\tresult = or_bits(result, s[i])\n    end for\n" &
	"    return result\nend function  -- or_all\n\n")
    puts(file, "global procedure not_found(sequence name)\n    puts(1, \"" &
	"Couldn't find \" & name & \"\\n\")\n    error = 1\nend procedure " &
	" -- not_found\n\n")
    puts(file, "global function link_c_func(atom dll, sequence name, sequence" &
	" args, atom result)\n-- link C routine as Euphoria function\n    " &
	"integer handle\n    handle = define_c_func(dll, name, args, result)" &
	"\n    if handle = -1 then\n\tnot_found(name)\n    end if\n    return" &
	" handle\nend function  -- link_c_func\n\n")
    puts(file, "global function link_c_proc(atom dll, sequence name, sequence" &
	" args)\n-- link C routine as Euphoria procedure\n    integer handle" &
	"\n    handle = define_c_proc(dll, name, args)\n    if handle = -1 " &
	"then\n\tnot_found(name)\n  end if\n    return handle\nend function " &
	" -- link_c_proc\n\n")
    puts(file, "global integer ")
    last = length(routine_list)
    line_len = 15
    put_list(routine_list)
    dlls = dll_list(routine_list)
    for i = 1 to length(dll_db) do
	if find(dll_db[i], dlls) then
	    last = length(constants_db[i])
	    if last then
		puts(file, "-- constants for " & dll_db[i] & ".dll\n")
		puts(file, "global constant ")
		line_len = 16
		put_list(constants_db[i])
	    end if
	    puts(file, "-- open " & dll_db[i] & ".dll\natom " & dll_db[i] &
		"\n" & dll_db[i] & " = open_dll(\"" & dll_db[i] & ".dll\")\n" &
		"if " & dll_db[i] & " = NULL then\n    not_found(\"" & 
		dll_db[i] & ".dll\")\nend if\n\n")
	end if
    end for
    puts(file, "-- link routines\n")
    for i = 1 to length(routine_db) do
	if find(routine_db[i][1], routine_list) then
	    printf(file, "%s = link_c_%s(%s, \"%s\", %s)\n", routine_db[i])
	    printf(file, "if %s = -1 then\n    not_found(\"%s\")\nend if\n", 
		    repeat(routine_db[i][1], 2))
	end if
    end for
    puts(file, "\nif error then\n    abort(1)\nend if\n-- end --\n")
    close(file)
    puts(1, "successfully generated " & file_name & " file!\n\n")
end procedure  -- dllsetup

-- version history
-- ---------------
-- version 1.0 - 3/19/98
-- version 1.1 - 3/27/98 - modified the database structure, sorted data

